from typing import List, Iterator, Dict

from bugzoo.core.bug import Bug
from bugzoo.core.coverage import FileLine

from .snippet import Snippet
from .source import SourceFile


class Transformation(object):
    @property
    def diff(self) -> str:
        raise NotImplementedError


class DeleteTransformation(object):
    def __init__(self, line: FileLine) -> None:
        self.__line = line

    @property
    def line(self) -> FileLine:
        return self.__line

    def __str__(self) -> str:
        return "DELETE[{}]".format(self.__line)


class ReplaceTransformation(object):
    """
    Replaces a numbered line in a given file with a provided snippet.
    """
    def __init__(self, line: FileLine, snippet: Snippet) -> None:
        self.__line = line
        self.__snippet = snippet

    @property
    def line(self) -> FileLine:
        return self.__line

    @property
    def snippet(self) -> Snippet:
        return self.__snippet

    def __str__(self) -> str:
        return "REPLACE[{}; {}]".format(self.__line, self.__snippet)


class AppendTransformation(object):
    """
    Appends a given snippet to a specific line in a given file.
    """
    def __init__(self, line: FileLine, snippet: Snippet) -> None:
        self.__line = line
        self.__snippet = snippet

    @property
    def snippet(self) -> Snippet:
        return self.__snippet

    @property
    def line(self) -> FileLine:
        return self.__line

    def __str__(self) -> str:
        return "APPEND[{}; {}]".format(self.__line, self.__snippet)



from typing import List


def get_lines(fn: str) -> List[str]:
    """
    Attempts to return a list of all the lines in a given source code file.
    """
    # try to decode the file using the default encoding (should be utf-8)
    try:
        with open(fn, 'r') as f:
            return [l.rstrip('\n') for l in f]
    except UnicodeDecodeError:
        pass

    # let's try to decode the using latin-1 encoding
    with open(fn, 'r', encoding='latin-1') as f:
        return [l.rstrip('\n') for l in f]
